/* Scrutari - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.api.EngineMetadata;
import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import java.net.URL;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.Phrase;


/**
 *
 * @author Vincent Calame
 */
public final class MetadataPrint {

    private MetadataPrint() {

    }

    public static void printMetadata(AdminHtmlPage hp, EngineMetadata engineMetadata) {
        boolean done = false;
        if (MetadataPrint.printLabelHolder(hp, "title", engineMetadata)) {
            done = true;
        }
        for (Phrase phrase : engineMetadata.getPhrases().getPhraseList()) {
            if (MetadataPrint.printLabelHolder(hp, "phrase=" + phrase.getName(), phrase)) {
                done = true;
            }
        }
        if (MetadataPrint.printUrl(hp, "icon", engineMetadata.getIcon())) {
            done = true;
        }
        if (MetadataPrint.printUrl(hp, "website", engineMetadata.getWebsite())) {
            done = true;
        }
        if (MetadataPrint.printAttributeHolder(hp, engineMetadata)) {
            done = true;
        }
        if (!done) {
            hp
                    .P()
                    .SPAN(HA.classes("scrutari-admin-Info"))
                    .localize("_ info.sctconf.none_metadata")
                    ._SPAN()
                    ._P();
        }
    }

    public static boolean printLabelHolder(AdminHtmlPage hp, String name, LabelHolder labelHolder) {
        int labelCount = labelHolder.getLabelCount();
        if (labelCount == 0) {
            return false;
        }
        hp
                .P()
                .CODE()
                .escape('[')
                .escape(name)
                .escape(']')
                ._CODE()
                ._P();
        hp
                .UL();
        for (int i = 0; i < labelCount; i++) {
            Label label = labelHolder.getLabelByIndex(i);
            hp
                    .LI()
                    .escape(label.getLang())
                    .escape(" = ")
                    .escape(label.getLabelString())
                    ._LI();
        }
        hp
                ._UL();
        return true;
    }

    public static boolean printUrl(AdminHtmlPage hp, String name, URL url) {
        if (url == null) {
            return false;
        }
        hp
                .P()
                .CODE()
                .escape(name)
                .escape(" = ")
                .A(HA.href(url.toString()))
                .escape(url.toString())
                ._A()
                ._CODE()
                ._P();
        return true;
    }

    public static boolean printAttributeHolder(AdminHtmlPage hp, AttributeHolder attributeHolder) {
        int count = attributeHolder.getAttributeCount();
        if (count == 0) {
            return false;
        }
        for (int i = 0; i < count; i++) {
            Attribute attribute = attributeHolder.getAttributeByIndex(i);
            hp
                    .P()
                    .CODE()
                    .escape(attribute.getAttributeKey())
                    .escape(" = ");
            int valueCount = attribute.getValueCount();
            for (int j = 0; j < valueCount; j++) {
                if (j > 0) {
                    hp
                            .escape(" ; ");
                }
                hp
                        .escape(attribute.getValue(j));
            }
            hp
                    ._CODE()
                    ._P();
        }
        return true;
    }

}
