/* ScrutariLib_DB - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */
package net.scrutari.db.tools.lexies;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.collation.CollationUnit;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;
import net.scrutari.lexie.ScrutariLexieUnit;

/**
 *
 * @author Vincent Calame
 */
public class LexieUnitBuilder {

    private final Map<Lang, CollatedKeyMap<Long>> lexieMapByLangMap = new HashMap<Lang, CollatedKeyMap<Long>>();
    private final Map<Long, ScrutariLexieUnit> lexieUnitMap = new LinkedHashMap<Long, ScrutariLexieUnit>();
    private long newId = 1;

    public LexieUnitBuilder() {
    }

    public Collection<ScrutariLexieUnit> getLexieUnitList() {
        return lexieUnitMap.values();
    }

    public LexieOccurrence getLexieOccurence(Lang lang, String sourceLexie, boolean inFiche) {
        CollatedKeyMap<Long> map = getOrCreateLexieMap(lang);
        CollationUnit collationUnit = map.collateString(sourceLexie);
        String collatedString = collationUnit.getCollatedString();
        Long lg = map.getValueByCollatedKey(collatedString);
        ScrutariLexieUnit scrutariLexieUnit;
        if (lg == null) {
            lg = newId;
            newId++;
            scrutariLexieUnit = new ScrutariLexieUnit(lg, lang, collatedString);
            scrutariLexieUnit.setCanonicalLexie(collationUnit.getSourceString());
            lexieUnitMap.put(lg, scrutariLexieUnit);
            map.putValueByCollatedKey(collatedString, lg);
        } else {
            scrutariLexieUnit = lexieUnitMap.get(lg);
        }
        if (inFiche) {
            scrutariLexieUnit.setHasFicheLink(true);
        } else {
            scrutariLexieUnit.setHasMotcleLink(true);
        }
        return new LexieOccurrence(lg, collationUnit);
    }

    public Map<Lang, CollatedKeyMap<Long>> getLexieMapByLangMap() {
        return lexieMapByLangMap;
    }

    private CollatedKeyMap<Long> getOrCreateLexieMap(Lang lang) {
        CollatedKeyMap<Long> collatedStringMap = lexieMapByLangMap.get(lang);
        if (collatedStringMap == null) {
            collatedStringMap = new SortedCollatedKeyMap<Long>(lang.toLocale());
            lexieMapByLangMap.put(lang, collatedStringMap);
        }
        return collatedStringMap;
    }

}
