/* ScrutariLib_Lexie - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.lexie;

import java.io.IOException;
import java.util.AbstractList;
import java.util.Collections;
import java.util.List;
import java.util.RandomAccess;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.collation.CollationUnitPosition;


/**
 *
 * @author Vincent Calame
 */
public class LexieUtils {

    public final static List<ScrutariLexieUnit> EMPTY_SCRUTARILEXIEUNITLIST = Collections.emptyList();
    public final static List<LexieOccurrences> EMPTY_LEXIEOCCURRENCESLIST = Collections.emptyList();

    private LexieUtils() {
    }

    public static void writeCollationUnitPosition(CollationUnitPosition collationUnitPosition, PrimitivesWriter primitivesWriter) throws IOException {
        SubstringPosition sp = collationUnitPosition.getPositionInWholeSource();
        primitivesWriter.writeInt(sp.getBeginIndex());
        primitivesWriter.writeInt(sp.getLength());
        int[] array = collationUnitPosition.getCorrespondances();
        int length = array.length;
        primitivesWriter.writeInt(length);
        for (int i = 0; i < length; i++) {
            primitivesWriter.writeInt(array[i]);
        }
    }

    public static CollationUnitPosition readCollationUnitPosition(PrimitivesReader primitivesReader) throws IOException {
        int beginIndex = primitivesReader.readInt();
        int length = primitivesReader.readInt();
        SubstringPosition sp = new SubstringPosition(beginIndex, length);
        int al = primitivesReader.readInt();
        int[] correspondances = new int[al];
        for (int i = 0; i < al; i++) {
            correspondances[i] = primitivesReader.readInt();
        }
        return new CollationUnitPosition(sp, correspondances);
    }

    public static List<ScrutariLexieUnit> toScrutariLexieUnitList(ScrutariLexieUnit[] scrutariLexieUnitArray) {
        if (scrutariLexieUnitArray == null) {
            return EMPTY_SCRUTARILEXIEUNITLIST;
        }
        if (scrutariLexieUnitArray.length == 0) {
            return EMPTY_SCRUTARILEXIEUNITLIST;
        }
        return new ScrutariLexieUnitList(scrutariLexieUnitArray);
    }

    public static List<ScrutariLexieUnit> toScrutariLexieUnitList(ScrutariLexieUnit scrutariLexieUnit) {
        if (scrutariLexieUnit == null) {
            return EMPTY_SCRUTARILEXIEUNITLIST;
        }
        return Collections.singletonList(scrutariLexieUnit);
    }

    public static List<LexieOccurrences> wrap(LexieOccurrences[] lexieOccurrencesArray) {
        if (lexieOccurrencesArray == null) {
            return EMPTY_LEXIEOCCURRENCESLIST;
        }
        if (lexieOccurrencesArray.length == 0) {
            return EMPTY_LEXIEOCCURRENCESLIST;
        }
        return new LexieOccurrencesList(lexieOccurrencesArray);
    }

    public static List<LexieOccurrences> wrap(LexieOccurrences lexieOccurrences) {
        if (lexieOccurrences == null) {
            return EMPTY_LEXIEOCCURRENCESLIST;
        }
        return Collections.singletonList(lexieOccurrences);
    }


    private static class LexieOccurrencesList extends AbstractList<LexieOccurrences> implements RandomAccess {

        private final LexieOccurrences[] array;

        private LexieOccurrencesList(LexieOccurrences[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public LexieOccurrences get(int index) {
            return array[index];
        }

    }


    private static class ScrutariLexieUnitList extends AbstractList<ScrutariLexieUnit> implements RandomAccess {

        private final ScrutariLexieUnit[] array;

        private ScrutariLexieUnitList(ScrutariLexieUnit[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public ScrutariLexieUnit get(int index) {
            return array[index];
        }

    }

}
