/* ScrutariLib_SearchEngine - Copyright (c) 2013-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.CanonicalQ;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.api.result.FicheSearchSource;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchSourceBuilder {

    private QId qId;
    private CanonicalQ canonicalQ;
    private SearchOptionsDef searchOptionsDef;

    public FicheSearchSourceBuilder() {
    }

    public void setQId(QId qId) {
        this.qId = qId;
    }

    public void setCanonicalQ(CanonicalQ canonicalQ) {
        this.canonicalQ = canonicalQ;
    }

    public void setSearchOptions(SearchOptionsDef searchOptionsDef) {
        this.searchOptionsDef = searchOptionsDef;
    }

    public boolean isValid() {
        return (canonicalQ != null);
    }

    public FicheSearchSource toFicheSearchSource() {
        if (canonicalQ == null) {
            throw new IllegalStateException("qInfo is not defined");
        }
        return new InternalFicheSearchSource(qId, canonicalQ, searchOptionsDef);
    }

    public static FicheSearchSource build(QId qId, CanonicalQ canonicalQ, SearchOptionsDef searchOptionsDef) {
        if (canonicalQ == null) {
            throw new IllegalArgumentException("qInfo is null");
        }
        return new InternalFicheSearchSource(qId, canonicalQ, searchOptionsDef);
    }


    private static class InternalFicheSearchSource implements FicheSearchSource {

        private final QId qId;
        private final CanonicalQ canonicalQ;
        private final SearchOptionsDef searchOptionsDef;

        private InternalFicheSearchSource(QId qId, CanonicalQ canonicalQ, SearchOptionsDef searchOptionsDef) {
            this.qId = qId;
            this.canonicalQ = canonicalQ;
            this.searchOptionsDef = searchOptionsDef;
        }

        @Override
        public QId getQId() {
            return qId;
        }

        @Override
        public CanonicalQ getCanonicalQ() {
            return canonicalQ;
        }

        @Override
        public SearchOptionsDef getSearchOptionsDef() {
            return searchOptionsDef;
        }

    }

}
