/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.tools.ErrorUtils;
import java.text.ParseException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.ChangeLabelHolderBuilder;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.PhrasesBuilder;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public final class ConfDOMUtils {

    private ConfDOMUtils() {

    }

    public static void readPhraseElement(PhrasesBuilder phrasesBuilder, Element element, MessageHandler messageHandler, String xpath) {
        String name = element.getAttribute("name");
        if (name.length() == 0) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
            return;
        }
        xpath = xpath + "[@name='" + name + "']";
        readPhraseElement(phrasesBuilder, element, messageHandler, xpath, name);
    }

    public static void readPhraseElement(PhrasesBuilder phrasesBuilder, Element element, MessageHandler messageHandler, String xpath, String name) {
        ChangeLabelHolderBuilder phraseBuilder = phrasesBuilder.getPhraseBuilder(name);
        PhraseElementHandler handler = new PhraseElementHandler(phraseBuilder, messageHandler, xpath);
        DOMUtils.readChildren(element, handler);
    }


    private static class PhraseElementHandler implements ElementHandler {

        private final ChangeLabelHolderBuilder phraseBuilder;
        private final MessageHandler messageHandler;
        private final String xpath;

        private PhraseElementHandler(ChangeLabelHolderBuilder phraseBuilder, MessageHandler messageHandler, String xpath) {
            this.phraseBuilder = phraseBuilder;
            this.messageHandler = messageHandler;
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String tagPath = xpath + "/" + tagName;
            if (tagName.equals("label")) {
                String langString = element.getAttribute("xml:lang");
                if (langString.isEmpty()) {
                    ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", tagPath, "xml:lang");
                } else {
                    tagPath = tagPath + "[@xml:lang='" + langString + "']";
                    try {
                        Lang lang = Lang.parse(element.getAttribute("xml:lang"));
                        CleanedString labelString = DOMUtils.contentToCleanedString(element);
                        if (labelString != null) {
                            phraseBuilder.putLabel(lang, labelString);
                        } else {
                            ErrorUtils.severe(messageHandler, "_ error.empty.xml.tag", tagPath, langString);
                        }
                    } catch (ParseException pe) {
                        ErrorUtils.severe(messageHandler, "_ error.wrong.xml.langattributevalue", tagPath, langString);
                    }
                }
            } else {
                ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", tagPath);
            }
        }

    }

}
