/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.tools.DataValidatorBuilder;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.ValidationDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.CategorizedMessageEntry;
import net.mapeadores.util.logging.SimpleMessageHandler;
import net.scrutari.data.DataValidator;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
class ValidationConf {

    private String processURI;
    private List<CategorizedMessageEntry> messageList;
    private DataValidator dataValidator = DataValidatorBuilder.NEUTRAL_DATAVALIDATOR;

    private ValidationConf() {

    }

    DataValidator getDataValidator() {
        return dataValidator;
    }

    static ValidationConf load(File f) throws IOException, ParserConfigurationException {
        ValidationConf validationConf = new ValidationConf();
        validationConf.init(f);
        return validationConf;
    }

    String getProcessURI() {
        return processURI;
    }

    List<CategorizedMessageEntry> getMessageList() {
        return messageList;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = f.getPath();
        SimpleMessageHandler simpleHandler = new SimpleMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.xml(simpleHandler, "_ error.exception.xml.sax", saxe.getMessage());
            messageList = simpleHandler.toList();
            return;
        }
        DataValidatorBuilder dataValidatorBuilder = new DataValidatorBuilder();
        ValidationDOMReader validationDOMReader = new ValidationDOMReader(dataValidatorBuilder, simpleHandler);
        validationDOMReader.readValidation(document.getDocumentElement(), "");
        dataValidator = dataValidatorBuilder.toDataValidator();
        messageList = simpleHandler.toList();
    }

}
