/* UtilLib - Copyright (c) 2010-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.util.HashMap;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class CommandBox {

    private String name;
    private String lockey;
    private Map<String, String> hiddenMap;
    private String page;
    private String errorPage;
    private String action;
    private String family;
    private String target;
    private boolean multipart = false;

    public CommandBox() {
    }

    private CommandBox(CommandBox commandBox, String name, String lockey) {
        this.name = name;
        this.lockey = lockey;
        if (commandBox.hiddenMap != null) {
            this.hiddenMap = new HashMap<String, String>(commandBox.hiddenMap);
        }
        this.page = commandBox.page;
        this.errorPage = commandBox.errorPage;
        this.action = commandBox.action;
        this.family = commandBox.family;
        this.target = commandBox.target;
        this.multipart = commandBox.multipart;
    }

    public CommandBox derive(String name, String lockey) {
        return new CommandBox(this, name, lockey);
    }

    public String name() {
        return name;
    }

    public CommandBox name(String name) {
        this.name = name;
        return this;
    }

    public String lockey() {
        return lockey;
    }

    public CommandBox lockey(String lockey) {
        this.lockey = lockey;
        return this;
    }

    public Map<String, String> hiddenMap() {
        return hiddenMap;
    }

    public CommandBox hidden(String name, String value) {
        if (hiddenMap == null) {
            hiddenMap = new HashMap<String, String>();
        }
        hiddenMap.put(name, value);
        return this;
    }

    public String page() {
        return page;
    }

    public CommandBox page(String page) {
        this.page = page;
        return this;
    }

    public String errorPage() {
        return errorPage;
    }

    public CommandBox errorPage(String errorPage) {
        this.errorPage = errorPage;
        return this;
    }

    public String action() {
        return action;
    }

    public CommandBox action(String action) {
        this.action = action;
        return this;
    }

    public String family() {
        return family;
    }

    public CommandBox family(String family) {
        this.family = family;
        return this;
    }

    public String target() {
        return target;
    }

    public CommandBox target(String target) {
        this.target = target;
        return this;
    }

    public boolean multipart() {
        return multipart;
    }

    public CommandBox multipart(boolean multipart) {
        this.multipart = multipart;
        return this;
    }

}
