/* UtilLib - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Set;


/**
 *
 * @author Vincent Calame
 */
public class ListLangContextBuilder {

    private final List<Lang> langList = new ArrayList<Lang>();
    private final List<Locale> localeList = new ArrayList<Locale>();
    private final Set<Lang> existingLang = new HashSet<Lang>();

    public ListLangContextBuilder() {

    }

    public void addLang(Lang lang) {
        if (!existingLang.contains(lang)) {
            existingLang.add(lang);
            langList.add(lang);
            localeList.add(lang.toLocale());
        }

    }

    public void addLang(Lang lang, Locale formatLocale) {
        if (!existingLang.contains(lang)) {
            existingLang.add(lang);
            langList.add(lang);
            localeList.add(formatLocale);
        }
    }

    public void addLangList(Langs langList) {
        int count = langList.getLangCount();
        for (int i = 0; i < count; i++) {
            addLang(langList.getLang(i));
        }
    }

    public ListLangContext toListLangContext() {
        int size = langList.size();
        if (size == 0) {
            throw new IllegalStateException("ListLangContext is empty");
        }
        Lang[] langArray = new Lang[size];
        Locale[] localeArray = new Locale[size];
        for (int i = 0; i < size; i++) {
            langArray[i] = langList.get(i);
            localeArray[i] = localeList.get(i);
        }
        return new InternalListLangContext(langArray, localeArray);
    }

    public static ListLangContext build(Lang[] langArray) {
        int length = langArray.length;
        if (length == 0) {
            throw new IllegalArgumentException("langIntegerArray.length == 0");
        }
        ListLangContextBuilder builder = new ListLangContextBuilder();
        for (int i = 0; i < length; i++) {
            builder.addLang(langArray[i]);
        }
        return builder.toListLangContext();
    }

    public static ListLangContext build(Langs langList) {
        if (langList.getLangCount() == 0) {
            throw new IllegalArgumentException("langList.getLangCount() == 0");
        }
        ListLangContextBuilder builder = new ListLangContextBuilder();
        builder.addLangList(langList);
        return builder.toListLangContext();
    }


    private static class InternalListLangContext implements ListLangContext {

        private final Lang[] langArray;
        private final Locale[] localeArray;

        private InternalListLangContext(Lang[] langArray, Locale[] localeArray) {
            this.langArray = langArray;
            this.localeArray = localeArray;
        }

        @Override
        public int getLangCount() {
            return langArray.length;
        }

        @Override
        public Lang getLang(int index) {
            return langArray[index];
        }

        @Override
        public Locale getFormatLocale(int index) {
            return localeArray[index];
        }

    }

}
