/* UtilLib - Copyright (c) 2008-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.SimpleTimeZone;


/**
 *
 * @author Vincent Calame
 */
public class TimeLog {

    private final static SimpleDateFormat ISO_FORMAT = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss,SSS'Z'");

    static {
        ISO_FORMAT.setTimeZone(new SimpleTimeZone(0, "GMT"));
    }

    private static String SEPARATEUR = "\t";
    private long startTime;
    private long previousTime;
    private StringBuilder buf;

    public TimeLog() {
        buf = new StringBuilder();
        startTime = System.currentTimeMillis();
        buf.append(ISO_FORMAT.format(new Date(startTime)));
        buf.append("\n\n");
        previousTime = startTime;
    }

    public void addSection(String section) {
        buf.append("[");
        buf.append(section);
        buf.append("]\n");
    }

    public void log(String key) {
        long currentTime = System.currentTimeMillis();
        buf.append(key);
        buf.append(SEPARATEUR);
        buf.append("+ ");
        printIntervalle(buf, currentTime, previousTime);
        buf.append(SEPARATEUR);
        printIntervalle(buf, currentTime, startTime);
        buf.append("\n\n");
        previousTime = currentTime;
    }

    @Override
    public String toString() {
        return buf.toString();
    }

    private static void printIntervalle(StringBuilder buf, long currentTime, long oldTime) {
        int diff = (int) (currentTime - oldTime);
        if (diff == 0) {
            buf.append("00:00");
        } else {
            double ms = ((double) diff) / 1000;
            double s = Math.floor(ms);
            ms = (ms - s) * 1000;
            if (s > 0) {
                double mn = Math.floor(s / 60);
                s = s - (mn * 60);
                buf.append(String.valueOf((int) mn));
            } else {
                buf.append('0');
            }
            buf.append(':');
            if (s < 10) {
                buf.append('0');
            }
            buf.append(String.valueOf((int) s));
            if (ms > 0) {
                buf.append(',');
                if (ms < 10) {
                    buf.append('0');
                }
                if (ms < 100) {
                    buf.append('0');
                }
                buf.append(String.valueOf((int) ms));
            }
        }
    }

}
