/* UtilLib - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class PhrasesBuilder {

    private final Map<String, ChangeLabelHolderBuilder> builderMap = new LinkedHashMap<String, ChangeLabelHolderBuilder>();

    public PhrasesBuilder() {

    }

    public ChangeLabelHolderBuilder getPhraseBuilder(String name) {
        ChangeLabelHolderBuilder builder = builderMap.get(name);
        if (builder == null) {
            builder = new ChangeLabelHolderBuilder();
            builderMap.put(name, builder);
        }
        return builder;
    }

    public Phrases toPhrases() {
        Map<String, Phrase> phraseMap = new HashMap<String, Phrase>();
        int size = builderMap.size();
        Phrase[] phraseArray = new Phrase[size];
        int p = 0;
        for (Map.Entry<String, ChangeLabelHolderBuilder> entry : builderMap.entrySet()) {
            String name = entry.getKey();
            Phrase phrase = entry.getValue().toPhrase(name);
            phraseMap.put(name, phrase);
            phraseArray[p] = phrase;
            p++;
        }
        List<Phrase> phraseList = StringUtils.wrap(phraseArray);
        return new InternalPhrases(phraseList, phraseMap);
    }


    private static class InternalPhrases implements Phrases {

        private final List<Phrase> phraseList;
        private final Map<String, Phrase> phraseMap;

        private InternalPhrases(List<Phrase> phraseList, Map<String, Phrase> phraseMap) {
            this.phraseList = phraseList;
            this.phraseMap = phraseMap;
        }

        @Override
        public List<Phrase> getPhraseList() {
            return phraseList;
        }

        @Override
        public Phrase getPhrase(String name) {
            return phraseMap.get(name);
        }

    }

}
